'use strict';

const kit = require('@nuxt/kit');
const estreeWalker = require('estree-walker');
const MagicString = require('magic-string');

const HELPER_NAME = "mockNuxtImport";
const autoImportMock = kit.defineNuxtModule({
  meta: {
    name: "vitest-env:auto-import-mock"
  },
  setup(_, nuxt) {
    let imports = [];
    nuxt.hook("imports:extend", (_2) => {
      imports = _2;
    });
    kit.addVitePlugin({
      name: "nuxt:auto-import-mock",
      transform(code, id) {
        if (!code.includes(HELPER_NAME))
          return;
        if (id.includes("/node_modules/"))
          return;
        let ast;
        try {
          ast = this.parse(code, {
            sourceType: "module",
            ecmaVersion: "latest",
            ranges: true
          });
        } catch (e) {
          return;
        }
        const mocks = [];
        estreeWalker.walk(ast, {
          enter: (node) => {
            if (node.type !== "CallExpression")
              return;
            const call = node;
            if (call.callee.type !== "Identifier" || call.callee.name !== HELPER_NAME) {
              return;
            }
            if (call.arguments.length !== 2) {
              return;
            }
            if (call.arguments[0].type !== "Literal") {
              return;
            }
            const name = call.arguments[0].value;
            const importItem = imports.find((_2) => name === (_2.as || _2.name));
            if (!importItem) {
              return this.error(`Cannot find import "${name}" to mock`);
            }
            mocks.push({
              name,
              import: importItem,
              factory: code.slice(
                call.arguments[1].range[0],
                call.arguments[1].range[1]
              ),
              start: call.range[0],
              end: call.range[1]
            });
          }
        });
        if (!mocks.length)
          return;
        const s = new MagicString(code);
        const mockMap = /* @__PURE__ */ new Map();
        for (const mock of mocks) {
          s.overwrite(mock.start, mock.end, "");
          if (!mockMap.has(mock.import.from)) {
            mockMap.set(mock.import.from, []);
          }
          mockMap.get(mock.import.from).push(mock);
        }
        const mockCode = [...mockMap.entries()].map(([from, mocks2]) => {
          const lines = [
            `vi.mock(${JSON.stringify(from)}, async () => {`,
            `  const mod = { ...await vi.importActual(${JSON.stringify(
              from
            )}) }`
          ];
          for (const mock of mocks2) {
            lines.push(
              `  mod[${JSON.stringify(mock.name)}] = (${mock.factory})()`
            );
          }
          lines.push(`  return mod`);
          lines.push(`})`);
          return lines.join("\n");
        }).join("\n");
        s.append('\nimport {vi} from "vitest";\n' + mockCode);
        return {
          code: s.toString(),
          map: s.generateMap()
        };
      }
    });
  }
});

const modules = kit.defineNuxtModule({
  meta: {
    name: "vitest-env"
  },
  async setup() {
    await kit.installModule(autoImportMock);
  }
});

module.exports = modules;
